/****************************************************************************
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** Permission is hereby granted, free of charge, to any person obtaining a copy
** of this software and associated documentation files (the "Software"), to deal
** in the Software without restriction, including without limitation the rights
** to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
** copies of the Software, and to permit persons to whom the Software is
** furnished to do so, subject to the following conditions:
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
**  USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

/*!
 * @file accel15.c
 * @brief Accel 15 Click Driver.
 */

#include "accel15.h"

/**
 * @brief Dummy data.
 * @details Definition of dummy data.
 */
#define DUMMY  0x00

/**
 * @brief Config file array.
 * @details Global array that stores the configuration file of BMA490L.
 */
const uint8_t bma490l_config_file[ ACCEL15_RD_WR_MAX_LEN ] = 
{
    0xc8, 0x2e, 0x00, 0x2e, 0xc8, 0x2e, 0x00, 0x2e, 0xc8, 0x2e, 0x00, 0x2e, 0xc8, 0x2e, 0x00, 0x2e, 
    0xc8, 0x2e, 0x00, 0x2e, 0xc8, 0x2e, 0x00, 0x2e, 0xc8, 0x2e, 0x00, 0x2e, 0x80, 0x2e, 0x58, 0x01, 
    0x80, 0x2e, 0x74, 0x02, 0xb0, 0xf0, 0x10, 0x30, 0x21, 0x2e, 0x16, 0xf0, 0x80, 0x2e, 0xeb, 0x00, 
    0x19, 0x50, 0x17, 0x52, 0x01, 0x42, 0x3b, 0x80, 0x41, 0x30, 0x01, 0x42, 0x3c, 0x80, 0x00, 0x2e, 
    0x01, 0x40, 0x01, 0x42, 0x21, 0x2e, 0xff, 0xaf, 0xb8, 0x2e, 0x9b, 0x95, 0x80, 0x2e, 0x18, 0x00, 
    0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 
    0x80, 0x2e, 0x18, 0x00, 0xfd, 0x2d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x01, 0x2e, 0x55, 0xf0, 0xc0, 0x2e, 0x21, 0x2e, 0x55, 0xf0, 0x80, 0x2e, 0x18, 0x00, 0xfd, 0x2d, 
    0xaa, 0x00, 0x05, 0x00, 0xaa, 0x00, 0x05, 0x00, 0x40, 0x48, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x98, 0x2e, 0x80, 0x00, 0x20, 0x26, 0x98, 0x2e, 0xef, 0x00, 0x10, 0x30, 0x21, 0x2e, 0x59, 0xf0, 
    0x98, 0x2e, 0x38, 0x00, 0x98, 0x2e, 0x7f, 0x01, 0x98, 0x2e, 0x8e, 0x01, 0x00, 0x2e, 0x00, 0x2e, 
    0xd0, 0x2e, 0x98, 0x2e, 0xce, 0x00, 0x01, 0x2e, 0x34, 0x00, 0x00, 0xb2, 0x0d, 0x2f, 0x00, 0x30, 
    0x21, 0x2e, 0x34, 0x00, 0x01, 0x50, 0x98, 0x2e, 0x13, 0x01, 0x01, 0x50, 0x03, 0x52, 0x98, 0x2e, 
    0x00, 0xb0, 0x01, 0x50, 0x05, 0x52, 0x98, 0x2e, 0x00, 0xb0, 0x98, 0x2e, 0x38, 0x00, 0xe6, 0x2d, 
    0x13, 0x52, 0x40, 0x30, 0x42, 0x40, 0x90, 0x0a, 0x42, 0x42, 0x58, 0x84, 0x07, 0x52, 0xa1, 0x42, 
    0x71, 0x3c, 0x09, 0x56, 0x83, 0x42, 0xa9, 0x84, 0x83, 0x32, 0x84, 0x40, 0x61, 0x08, 0x4b, 0x0a, 
    0x81, 0x42, 0x82, 0x82, 0x02, 0x3f, 0x43, 0x40, 0x9a, 0x08, 0x52, 0x42, 0x40, 0x42, 0x7e, 0x80,
    0x61, 0x30, 0x01, 0x42, 0x10, 0x50, 0x01, 0x2e, 0x40, 0xf0, 0x1a, 0x90, 0xfb, 0x7f, 0x20, 0x2f, 
    0x03, 0x30, 0x0d, 0x50, 0x34, 0x33, 0x06, 0x30, 0x11, 0x52, 0x0b, 0x54, 0x55, 0x32, 0x1d, 0x1a, 
    0xe3, 0x22, 0x18, 0x1a, 0x0f, 0x58, 0xe3, 0x22, 0x04, 0x30, 0xd5, 0x40, 0xb5, 0x0d, 0xe1, 0xbe, 
    0x6f, 0xbb, 0x80, 0x91, 0xa9, 0x0d, 0x01, 0x89, 0xb5, 0x23, 0x10, 0xa1, 0xf7, 0x2f, 0xda, 0x0e, 
    0x34, 0x33, 0xeb, 0x2f, 0x01, 0x2e, 0x25, 0x00, 0x70, 0x1a, 0x00, 0x30, 0x21, 0x30, 0x02, 0x2c, 
    0x08, 0x22, 0x30, 0x30, 0x00, 0xb2, 0x06, 0x2f, 0x21, 0x2e, 0x59, 0xf0, 0x98, 0x2e, 0x38, 0x00, 
    0x00, 0x2e, 0x00, 0x2e, 0xd0, 0x2e, 0xfb, 0x6f, 0xf0, 0x5f, 0xb8, 0x2e, 0x1d, 0x50, 0x05, 0x2e, 
    0x00, 0xf0, 0x17, 0x56, 0xd3, 0x0f, 0x01, 0x40, 0xf4, 0x33, 0xcc, 0x08, 0x0d, 0x2f, 0xf4, 0x30, 
    0x94, 0x08, 0xb9, 0x88, 0x02, 0xa3, 0x04, 0x2f, 0x1b, 0x58, 0x4c, 0x0a, 0x87, 0xa2, 0x05, 0x2c, 
    0xcb, 0x22, 0x17, 0x54, 0x4a, 0x0a, 0xf2, 0x3b, 0xca, 0x08, 0x3c, 0x80, 0x27, 0x2e, 0x59, 0xf0, 
    0x01, 0x40, 0x01, 0x42, 0xb8, 0x2e, 0x1a, 0x24, 0x26, 0x00, 0x80, 0x2e, 0x58, 0x00, 0x00, 0x31, 
    0xc0, 0x2e, 0x21, 0x2e, 0xba, 0xf0, 0x12, 0x30, 0x12, 0x42, 0x02, 0x30, 0x12, 0x42, 0x12, 0x42, 
    0x12, 0x42, 0x02, 0x42, 0x03, 0x80, 0x41, 0x84, 0x11, 0x42, 0x02, 0x42, 0xb8, 0x2e, 0x44, 0x47, 
    0x35, 0x00, 0x46, 0x00, 0x4f, 0x00, 0xaf, 0x00, 0xff, 0x00, 0xff, 0xb7, 0x00, 0x02, 0x00, 0xb0, 
    0x05, 0x80, 0xb1, 0xf0, 0x88, 0x00, 0x80, 0x00, 0x5e, 0xf0, 0xc0, 0x00, 0x59, 0xf0, 0x89, 0xf0, 
    0x38, 0x00, 0x40, 0x00, 0x42, 0x00, 0x60, 0x50, 0x03, 0x2e, 0x45, 0x00, 0xe0, 0x7f, 0xf1, 0x7f, 
    0xdb, 0x7f, 0x30, 0x30, 0x15, 0x54, 0x0a, 0x1a, 0x28, 0x2f, 0x1a, 0x25, 0x7a, 0x82, 0x00, 0x30, 
    0x43, 0x30, 0x32, 0x30, 0x05, 0x30, 0x04, 0x30, 0xf6, 0x6f, 0xf2, 0x09, 0xfc, 0x13, 0xc2, 0xab, 
    0xb3, 0x09, 0xef, 0x23, 0x80, 0xb3, 0xe6, 0x6f, 0xb7, 0x01, 0x00, 0x2e, 0x8b, 0x41, 0x4b, 0x42,
    0x03, 0x2f, 0x46, 0x40, 0x86, 0x17, 0x81, 0x8d, 0x46, 0x42, 0x41, 0x8b, 0x23, 0xbd, 0xb3, 0xbd, 
    0x03, 0x89, 0x41, 0x82, 0x07, 0x0c, 0x43, 0xa3, 0xe6, 0x2f, 0xe1, 0x6f, 0xa2, 0x6f, 0x52, 0x42, 
    0x00, 0x2e, 0xb2, 0x6f, 0x52, 0x42, 0x00, 0x2e, 0xc2, 0x6f, 0x42, 0x42, 0x03, 0xb2, 0x06, 0x2f, 
    0x01, 0x2e, 0x59, 0xf0, 0x01, 0x32, 0x01, 0x0a, 0x21, 0x2e, 0x59, 0xf0, 0x06, 0x2d, 0x01, 0x2e, 
    0x59, 0xf0, 0xf1, 0x3d, 0x01, 0x08, 0x21, 0x2e, 0x59, 0xf0, 0xdb, 0x6f, 0xa0, 0x5f, 0xb8, 0x2e, 
    0x60, 0x50, 0xc3, 0x7f, 0xd4, 0x7f, 0xe7, 0x7f, 0xf6, 0x7f, 0xb2, 0x7f, 0xa5, 0x7f, 0x36, 0x30, 
    0x07, 0x2e, 0x01, 0xf0, 0xbe, 0xbd, 0xbe, 0xbb, 0x1f, 0x58, 0x77, 0x05, 0x01, 0x56, 0x21, 0x54, 
    0x27, 0x41, 0x06, 0x41, 0xf8, 0xbf, 0xbe, 0x0b, 0xb5, 0x11, 0xd6, 0x42, 0x03, 0x89, 0x5a, 0x0e, 
    0xf6, 0x2f, 0x12, 0x30, 0x25, 0x2e, 0x34, 0x00, 0x02, 0x31, 0x25, 0x2e, 0xb8, 0xf0, 0xd4, 0x6f, 
    0xc3, 0x6f, 0xe7, 0x6f, 0xb2, 0x6f, 0xa5, 0x6f, 0xf6, 0x6f, 0xa0, 0x5f, 0xc8, 0x2e, 0x10, 0x50, 
    0x23, 0x52, 0x03, 0x50, 0xfb, 0x7f, 0x98, 0x2e, 0xf3, 0x00, 0x03, 0x52, 0x45, 0x82, 0x10, 0x30, 
    0x50, 0x42, 0x60, 0x30, 0xfb, 0x6f, 0xc0, 0x2e, 0x40, 0x42, 0xf0, 0x5f, 0x10, 0x50, 0x25, 0x52, 
    0x05, 0x50, 0xfb, 0x7f, 0x98, 0x2e, 0xf3, 0x00, 0x05, 0x52, 0x45, 0x82, 0x00, 0x30, 0x50, 0x42, 
    0x70, 0x30, 0xfb, 0x6f, 0xc0, 0x2e, 0x40, 0x42, 0xf0, 0x5f, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e,
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e,
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 
    0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0x80, 0x2e, 0x18, 0x00, 0xfd, 0x2d, 
    0x46, 0x86, 0x70, 0x50, 0xe5, 0x40, 0xc3, 0x88, 0x42, 0x84, 0x04, 0x41, 0xc3, 0x40, 0x06, 0x41, 
    0x6d, 0xbb, 0xc2, 0x7f, 0xf5, 0x7f, 0x80, 0xb3, 0xe6, 0x7f, 0xd0, 0x7f, 0xb3, 0x7f, 0x12, 0x30, 
    0x5e, 0x2f, 0x31, 0x25, 0x55, 0x40, 0x41, 0x91, 0xa1, 0x7f, 0x0f, 0x2f, 0x01, 0x30, 0xc1, 0x42, 
    0x00, 0x2e, 0xc2, 0x6f, 0x13, 0x40, 0x93, 0x42, 0x00, 0x2e, 0x13, 0x40, 0x93, 0x42, 0x00, 0x2e, 
    0x00, 0x40, 0x80, 0x42, 0xbd, 0x80, 0xc0, 0x2e, 0x01, 0x42, 0x90, 0x5f, 0xc7, 0x86, 0x01, 0x30, 
    0xc5, 0x40, 0xfb, 0x86, 0x45, 0x41, 0x04, 0x41, 0x43, 0xbe, 0xc3, 0xbb, 0xd5, 0xbe, 0x55, 0xba, 
    0x97, 0x7f, 0x05, 0x30, 0xd1, 0x15, 0xf7, 0x09, 0xc0, 0xb3, 0x09, 0x2f, 0x06, 0x40, 0xc7, 0x40, 
    0xb7, 0x05, 0x07, 0x30, 0x80, 0xa9, 0xfe, 0x05, 0xb7, 0x23, 0x74, 0x0f, 0x55, 0x23, 0xe6, 0x6f, 
    0x41, 0x82, 0x01, 0x80, 0xc1, 0x86, 0x43, 0xa2, 0xec, 0x2f, 0xb0, 0x6f, 0xa4, 0x6f, 0x28, 0x1a, 
    0xd1, 0x6f, 0xc3, 0x6f, 0x02, 0x2f, 0x02, 0x30, 0x18, 0x2c, 0x02, 0x43, 0x05, 0x41, 0x6a, 0x29, 
    0x96, 0x6f, 0x05, 0x43, 0x6e, 0x0e, 0x10, 0x2f, 0xf4, 0x6f, 0x00, 0xb3, 0x03, 0x2f, 0x3f, 0x89, 
    0x94, 0x14, 0x25, 0x2e, 0x5e, 0xf0, 0x41, 0x25, 0x23, 0x25, 0x15, 0x41, 0x95, 0x42, 0x00, 0x2e,
    0x15, 0x41, 0x95, 0x42, 0x00, 0x2e, 0x04, 0x41, 0x84, 0x42, 0x00, 0x90, 0x09, 0x2f, 0x50, 0x40, 
    0xd0, 0x42, 0x00, 0x2e, 0x50, 0x40, 0xd0, 0x42, 0x00, 0x2e, 0x40, 0x40, 0x02, 0x2c, 0xc0, 0x42, 
    0x42, 0x42, 0x90, 0x5f, 0xb8, 0x2e, 0x00, 0x2e, 0x10, 0x24, 0x8a, 0x02, 0x11, 0x24, 0x00, 0x0c, 
    0x12, 0x24, 0x80, 0x2e, 0x13, 0x24, 0x18, 0x00, 0x12, 0x42, 0x13, 0x42, 0x41, 0x1a, 0xfb, 0x2f, 
    0x10, 0x24, 0x50, 0x39, 0x11, 0x24, 0x21, 0x2e, 0x21, 0x2e, 0x10, 0x00, 0x23, 0x2e, 0x11, 0x00, 
    0x80, 0x2e, 0x10, 0x00,
};

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

/**
 * @brief Accel 15 configuration delay function.
 * @details Configuration delay for 100 milliseconds.
 *
 * @note None.
 */

static void accel15_config_delay ( void );

/**
 * @brief Accel 15 I2C writing function.
 * @details This function writes a desired number of data bytes starting from
 * the selected register by using I2C serial interface.
 * @param[in] ctx : Click context object.
 * See #accel15_t object definition for detailed explanation.
 * @param[in] reg : Start register address.
 * @param[in] data_in : Data to be written.
 * @param[in] len : Number of bytes to be written.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t accel15_i2c_write ( accel15_t *ctx, uint8_t reg, uint8_t *data_in, uint8_t len );

/**
 * @brief Accel 15 I2C reading function.
 * @details This function reads a desired number of data bytes starting from
 * the selected register by using I2C serial interface.
 * @param[in] ctx : Click context object.
 * See #accel15_t object definition for detailed explanation.
 * @param[in] reg : Start register address.
 * @param[out] data_out : Output read data.
 * @param[in] len : Number of bytes to be read.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t accel15_i2c_read ( accel15_t *ctx, uint8_t reg, uint8_t *data_out, uint8_t len );

/**
 * @brief Accel 15 SPI writing function.
 * @details This function writes a desired number of data bytes starting from
 * the selected register by using SPI serial interface.
 * @param[in] ctx : Click context object.
 * See #accel15_t object definition for detailed explanation.
 * @param[in] reg : Start register address.
 * @param[in] data_in : Data to be written.
 * @param[in] len : Number of bytes to be written.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t accel15_spi_write ( accel15_t *ctx, uint8_t reg, uint8_t *data_in, uint8_t len );

/**
 * @brief Accel 15 SPI reading function.
 * @details This function reads a desired number of data bytes starting from
 * the selected register by using SPI serial interface.
 * @param[in] ctx : Click context object.
 * See #accel15_t object definition for detailed explanation.
 * @param[in] reg : Start register address.
 * @param[out] data_out : Output read data.
 * @param[in] len : Number of bytes to be read.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t accel15_spi_read ( accel15_t *ctx, uint8_t reg, uint8_t *data_out, uint8_t len );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void accel15_cfg_setup ( accel15_cfg_t *cfg ) 
{
    cfg->scl  = HAL_PIN_NC;
    cfg->sda  = HAL_PIN_NC;
    cfg->sck  = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs   = HAL_PIN_NC;
    cfg->in2  = HAL_PIN_NC;
    cfg->in1  = HAL_PIN_NC;

    cfg->i2c_speed   = I2C_MASTER_SPEED_STANDARD;
    cfg->i2c_address = ACCEL15_SET_DEV_ADDR_GND;

    cfg->spi_speed   = 100000;
    cfg->spi_mode    = SPI_MASTER_MODE_0;
    cfg->cs_polarity = SPI_MASTER_CHIP_SELECT_POLARITY_ACTIVE_LOW;

    cfg->drv_sel = ACCEL15_DRV_SEL_I2C;
}

void accel15_drv_interface_selection ( accel15_cfg_t *cfg, accel15_drv_t drv_sel ) 
{
    cfg->drv_sel = drv_sel;
}

err_t accel15_init ( accel15_t *ctx, accel15_cfg_t *cfg ) 
{
    ctx->drv_sel = cfg->drv_sel;

    if ( ACCEL15_DRV_SEL_I2C == ctx->drv_sel ) 
    {
        i2c_master_config_t i2c_cfg;

        i2c_master_configure_default( &i2c_cfg );

        i2c_cfg.scl = cfg->scl;
        i2c_cfg.sda = cfg->sda;

        ctx->slave_address = cfg->i2c_address;

        if ( I2C_MASTER_ERROR == i2c_master_open( &ctx->i2c, &i2c_cfg ) ) 
        {
            return I2C_MASTER_ERROR;
        }

        if ( I2C_MASTER_ERROR == i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address ) ) 
        {
            return I2C_MASTER_ERROR;
        }

        if ( I2C_MASTER_ERROR == i2c_master_set_speed( &ctx->i2c, cfg->i2c_speed ) ) 
        {
            return I2C_MASTER_ERROR;
        }

        ctx->read_f  = accel15_i2c_read;
        ctx->write_f = accel15_i2c_write;
    } 
    else 
    {
        spi_master_config_t spi_cfg;

        spi_master_configure_default( &spi_cfg );

        spi_cfg.sck  = cfg->sck;
        spi_cfg.miso = cfg->miso;
        spi_cfg.mosi = cfg->mosi;

        ctx->chip_select = cfg->cs;

        if ( SPI_MASTER_ERROR == spi_master_open( &ctx->spi, &spi_cfg ) ) 
        {
            return SPI_MASTER_ERROR;
        }

        if ( SPI_MASTER_ERROR == spi_master_set_default_write_data( &ctx->spi, DUMMY ) ) 
        {
            return SPI_MASTER_ERROR;
        }

        if ( SPI_MASTER_ERROR == spi_master_set_mode( &ctx->spi, cfg->spi_mode ) ) 
        {
            return SPI_MASTER_ERROR;
        }

        if ( SPI_MASTER_ERROR == spi_master_set_speed( &ctx->spi, cfg->spi_speed ) ) 
        {
            return SPI_MASTER_ERROR;
        }

        spi_master_set_chip_select_polarity( cfg->cs_polarity );
        spi_master_deselect_device( ctx->chip_select );

        ctx->read_f  = accel15_spi_read;
        ctx->write_f = accel15_spi_write;
    }

    digital_in_init( &ctx->in2, cfg->in2 );
    digital_in_init( &ctx->in1, cfg->in1 );

    return ACCEL15_OK;
}

err_t accel15_default_cfg ( accel15_t *ctx ) 
{
    err_t error_flag = ACCEL15_OK;
    uint8_t data_buf[ 128 ] = { 0 };
    uint16_t cnt = 0;
    uint16_t len = 0;
    uint8_t tx_buf[ 1 ] = { 0 };
    if ( ACCEL15_DRV_SEL_SPI == ctx->drv_sel ) 
    {
        tx_buf[ 0 ] = ACCEL15_NV_DISABLE_I2C;
        
        error_flag |= accel15_generic_write( ctx, ACCEL15_REG_NV_CONFIG, tx_buf, 1 );
        accel15_config_delay( );    
    }
    
    tx_buf[ 0 ] = ACCEL15_SOFT_RESET;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_CMD, tx_buf, 1 );
    accel15_config_delay( );

    tx_buf[ 0 ] = ACCEL15_APS_OFF | ACCEL15_FSW_ON;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_POWER_CONF, tx_buf, 1 );
    accel15_config_delay( );
    
    tx_buf[ 0 ] = ACCEL15_INIT_START;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_INIT_CTRL, tx_buf, 1 );
    accel15_config_delay( );
    
    len = sizeof ( bma490l_config_file );
    while ( cnt < len )
    {
        data_buf[ 0 ] = ( ( cnt / 2 ) & 0x0F );
        error_flag |= accel15_generic_write( ctx, ACCEL15_REG_RESERVED_REG_5B, data_buf, 1 );
        data_buf[ 0 ] = ( uint8_t ) ( ( cnt / 2 ) >> 4 );
        error_flag |= accel15_generic_write( ctx, ACCEL15_REG_RESERVED_REG_5C, data_buf, 1 );
        if ( ( len - cnt ) >= 128 ) 
        {
            memcpy ( data_buf, &bma490l_config_file[ cnt ], 128 );
            error_flag |= accel15_generic_write( ctx, ACCEL15_REG_FEATURE_CONFIG, data_buf, 128 );
        }
        else
        {
            memcpy ( data_buf, &bma490l_config_file[ cnt ], len - cnt );
            error_flag |= accel15_generic_write( ctx, ACCEL15_REG_FEATURE_CONFIG, data_buf, len - cnt );
        }
        cnt += 128;
    }
    accel15_config_delay( );
    
    tx_buf[ 0 ] = ACCEL15_INIT_STOP;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_INIT_CTRL, tx_buf, 1 );
    accel15_config_delay( );
    
    tx_buf[ 0 ] = ACCEL15_INT1_IO_CTRL_OUTPUT_EN | ACCEL15_INT1_IO_CTRL_LVL_HIGH;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_INT1_IO_CTRL, tx_buf, 1 );
    accel15_config_delay( );
    
    tx_buf[ 0 ] = ACCEL15_INT_MAP_DATA_INT1_DRDY;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_INT_MAP_DATA, tx_buf, 1 );
    accel15_config_delay( );
    
    accel15_cfg_accel( ctx, ACCEL15_OUTPUT_DATA_RATE_12_5HZ, ACCEL15_ACCEL_RANGE_2G );
    accel15_config_delay( );
    
    tx_buf[ 0 ] = ACCEL15_ACCEL_ON;
    error_flag |= accel15_generic_write( ctx, ACCEL15_REG_POWER_CTRL, tx_buf, 1 );
    accel15_config_delay( );
    return error_flag;
}

err_t accel15_generic_write ( accel15_t *ctx, uint8_t reg, uint8_t *data_in, uint8_t len ) 
{
    return ctx->write_f( ctx, reg, data_in, len );
}

err_t accel15_generic_read ( accel15_t *ctx, uint8_t reg, uint8_t *data_out, uint8_t len ) 
{
    return ctx->read_f( ctx, reg, data_out, len );
}

void accel15_soft_reset ( accel15_t *ctx ) 
{
    uint8_t tx_buf[ 1 ] = { 0 };
    
    tx_buf[ 0 ] = ACCEL15_SOFT_RESET;
    
    accel15_generic_write( ctx, ACCEL15_REG_CMD, tx_buf, 1 );
}

err_t accel15_check_id ( accel15_t *ctx ) 
{
    uint8_t id_val = 0;
    err_t return_flag = ACCEL15_OK;
    
    return_flag |= accel15_generic_read( ctx, ACCEL15_REG_CHIP_ID, &id_val, 1 );
    
    if ( ACCEL15_CHIP_ID != id_val ) 
    {
        return_flag = ACCEL15_ERROR;
    }
   
   return return_flag; 
}

uint8_t accel15_get_error ( accel15_t *ctx ) 
{
    uint8_t error = 0;
    
    accel15_generic_read( ctx, ACCEL15_REG_ERROR, &error, 1 );
    
    return error; 
}

uint8_t accel15_get_status ( accel15_t *ctx ) 
{
    uint8_t status = 0;
    
    accel15_generic_read( ctx, ACCEL15_REG_STATUS, &status, 1 );
    
    return status; 
}

void accel15_enable_accel ( accel15_t *ctx ) 
{
    uint8_t buf[ 1 ] = { 0 };
    
    accel15_generic_read( ctx, ACCEL15_REG_POWER_CTRL, buf, 1 );
    
    buf[ 0 ] &= ( ~ACCEL15_X_AXIS_EN );
    buf[ 0 ] |= ACCEL15_X_AXIS_EN;
    
    accel15_generic_write( ctx, ACCEL15_REG_POWER_CTRL, buf, 1 );
}

void accel15_cfg_accel ( accel15_t *ctx, uint8_t acc_odr, uint8_t g_range ) 
{
    uint8_t buf[ 1 ] = { 0 };
    
    accel15_generic_read( ctx, ACCEL15_REG_ACCEL_CONFIG, buf, 1 );
    
    buf[ 0 ] &= ( ~ACCEL15_ACCEL_ODR_MSK );
    buf[ 0 ] |= ( acc_odr & ACCEL15_ACCEL_ODR_MSK );
    
    accel15_generic_write( ctx, ACCEL15_REG_ACCEL_CONFIG, buf, 1 );
    
    accel15_generic_read( ctx, ACCEL15_REG_ACCEL_RANGE, buf, 1 );
    
    buf[ 0 ] &= ( ~ACCEL15_ACCEL_RANGE_MSK );
    buf[ 0 ] |= ( g_range & ACCEL15_ACCEL_RANGE_MSK );
    
    accel15_generic_write( ctx, ACCEL15_REG_ACCEL_RANGE, buf, 1 );
}

uint8_t accel15_get_internal_status ( accel15_t *ctx ) 
{
    uint8_t int_status = 0;
    
    accel15_generic_read( ctx, ACCEL15_REG_INTERNAL_STAT, &int_status, 1 );
    
    int_status &= ACCEL15_CONFIG_STREAM_MESSAGE_MSK;
    
    return int_status; 
}

err_t accel15_get_axis_data ( accel15_t *ctx, accel15_axis_t *axis ) 
{
    uint8_t rx_buf[ 6 ] = { 0 };
    uint16_t tmp = 0;
    
    err_t error_flag = accel15_generic_read( ctx, ACCEL15_REG_DATA_8, rx_buf, 6 );
    
    tmp = rx_buf[ 1 ];
    tmp <<= 8;
    tmp |= rx_buf[ 0 ];
    axis->x = ( int16_t ) tmp;
    
    tmp = rx_buf[ 3 ];
    tmp <<= 8;
    tmp |= rx_buf[ 2 ];
    axis->y = ( int16_t ) tmp;
    
    tmp = rx_buf[ 5 ];
    tmp <<= 8;
    tmp |= rx_buf[ 4 ];
    axis->z = ( int16_t ) tmp;
    
    return error_flag;
}

uint8_t accel15_get_int_1 ( accel15_t *ctx ) 
{
    return digital_in_read( &ctx->in1 );
}

uint8_t accel15_get_int_2 ( accel15_t *ctx ) 
{
    return digital_in_read( &ctx->in2 );
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void accel15_config_delay ( void )
{
    Delay_100ms( );
}

static err_t accel15_i2c_write ( accel15_t *ctx, uint8_t reg, uint8_t *data_in, uint8_t len ) 
{
    uint8_t tx_buf[ 256 ] = { 0 };
    tx_buf[ 0 ] = reg;
    for ( uint8_t cnt = 0; cnt < len; cnt++ ) 
    {
        tx_buf[ cnt + 1 ] = data_in[ cnt ];
    }
    return i2c_master_write( &ctx->i2c, tx_buf, len + 1 );
}

static err_t accel15_i2c_read ( accel15_t *ctx, uint8_t reg, uint8_t *data_out, uint8_t len ) 
{
    return i2c_master_write_then_read( &ctx->i2c, &reg, 1, data_out, len );
}

static err_t accel15_spi_write ( accel15_t *ctx, uint8_t reg, uint8_t *data_in, uint8_t len ) 
{
    uint8_t tx_buf[ 1 ] = { 0 };
    tx_buf[ 0 ] = reg;
    tx_buf[ 0 ] |= ACCEL15_SPI_WRITE;
    spi_master_select_device( ctx->chip_select );
    err_t error_flag = spi_master_write( &ctx->spi, tx_buf, 1 );
    error_flag |= spi_master_write( &ctx->spi, data_in, len );
    spi_master_deselect_device( ctx->chip_select );
    return error_flag;
}

static err_t accel15_spi_read ( accel15_t *ctx, uint8_t reg, uint8_t *data_out, uint8_t len ) 
{
    uint8_t tx_buf[ 2 ] = { 0 };
    tx_buf[ 0 ] = reg;
    tx_buf[ 0 ] |= ACCEL15_SPI_READ;
    tx_buf[ 1 ] = DUMMY;
    spi_master_select_device( ctx->chip_select );
    err_t error_flag = spi_master_write_then_read( &ctx->spi, tx_buf, 2, data_out, len );
    spi_master_deselect_device( ctx->chip_select );
    return error_flag;
}

// ------------------------------------------------------------------------ END
